#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Apr  9 14:20:37 2020

@author: fredodurand
"""

import matplotlib.pyplot as plt
import random
import math

# 



def simulation_1(beta, initial_I=100, duration = 60):

    I = [initial_I]
   
    for t in range(duration):
        new_infected = beta*I[-1]
        I.append(I[-1] + new_infected)

    return I


# I = simulation_1(0.5)
# plt.plot(I)

def simulation_2(beta, initial_I=100, population = 327e6, duration = 60):

    I = [initial_I]
    S = [population-initial_I]
    
    for t in range(duration):
        new_infected = beta*I[-1]*S[-1]/population
        I.append( I[-1] + new_infected )
        S.append( S[-1]- new_infected )
        
    return S, I

# S, I = simulation_2(0.5)
# plt.plot(I)
# plt.plot(S)


def simulation_sir(beta, gamma, initial_I=1000, population = 327e6, duration = 365):

    I = [initial_I]
    S = [population - initial_I]
    R = [0]   
   
    for t in range(duration):
        new_infected = beta*I[-1]*S[-1] / population
        new_recovered = gamma * I[-1]
        I.append( I[-1] + new_infected - new_recovered )
        S.append( S[-1] - new_infected )
        R.append( R[-1] + new_recovered )
         
    return S, I, R

beta = 0.5
gamma = 1/5

# S, I, R = simulation_sir(beta, gamma, duration = 100)

# plt.plot(S)
# plt.plot(I)
# plt.plot(R)

min_R0 = 2
max_R0 = 3
min_D = 4
max_D = 8

# for i in range(20):
#     gamma = 1 / (min_D + random.random()*(max_D-min_D))
#     beta = gamma* (min_R0 + random.random()*(max_R0-min_R0))
#     S, I, R = simulation_sir(beta, gamma)
#     plt.plot(I)
#     plt.plot(R)


min_R0 = 1.5
max_R0 = 4

# rate of growth at beginning of epidemic: beta - gamma = k
# R0 gamma -gamma = k
# gamma = k/(R0-1)

# k = 1/5*(2.5-1)
# random.seed(1)
# for i in range(20):    
#     R0 = min_R0 + random.random()*(max_R0-min_R0)
#     gamma = k /(R0-1)
#     beta = R0 * gamma
#     print (R0, 1/gamma)
#     S, I, R = simulation_sir(beta, gamma, duration = 45)
#     plt.plot(I)
#     #plt.plot(R)




def simulation_seir(beta, alpha, gamma, initial_I=100, population = 327e6, duration = 365):

    I = [initial_I]
    E = [0]
    S = [population - initial_I]
    R = [0]   
   
    for t in range(duration):
        new_exposed = beta*I[-1]*S[-1] / population
        new_infected = E[-1]*alpha
        new_recovered = gamma * I[-1]
        E.append( E[-1]+ new_exposed - new_infected )
        I.append( I[-1] + new_infected - new_recovered )
        S.append( S[-1] - new_exposed )
        R.append( R[-1] + new_recovered )
         
    return S, E, I, R

alpha = 1/4
beta = 0.5
gamma = 1/5

# S, E, I, R = simulation_seir(beta, alpha, gamma)

# plt.plot(S)
# plt.plot(E)
# plt.plot(I)
# plt.plot(R)




def simulation_sir_birth(beta, gamma, mu=1/50/365, initial_I=100/327e6, duration = 20*365, dt=0.1):
    
    S =[ 1.0 - initial_I] 
    I =[initial_I] 
    R =[0.0] 
   
    for t in range(int(duration/dt)):
        new_infected = beta*I[-1]*S[-1]
        new_recovered = gamma * I[-1]

        I.append(I[-1] + (new_infected - new_recovered -  mu*I[-1])*dt)
        S.append(S[-1] + (-new_infected + mu - mu*S[-1])*dt)
        R.append(R[-1] + (new_recovered-mu*R[-1])*dt)
        
        #print (S[-1], I[-1], R[-1], new_infected, new_recovered, mu)
 
    return S, I, R

R0 = 17.0
gamma = 1/5
beta = R0*gamma

# L_S, L_I, L_R = simulation_sir_birth(beta, gamma)

# plt.plot(L_S)
# plt.plot(L_I)
# plt.plot(L_R)


min_R0 = 1.1
max_R0 = 3
min_duration = 1
max_duration = 5

# add seasons


L_col =['tab:blue', "tab:orange", "tab:green", "tab:red", "tab:purple", "tab:brown", "tab:pink", "tab:gray", ]


# for i in range(120):
#     R0 = min_R0 + random.random()*(max_R0-min_R0)
#     L_max, L_tot = [], []
#     for j in range(40):
#         duration = min_duration + random.random()*(max_duration-min_duration)
#         gamma = 1/duration
#         beta = R0*gamma    
        
#         L_S, L_I, L_R = simulation_sir(beta, gamma)

#         L_max.append(max(L_I))
#         L_tot.append(L_R[-1])
#         #plt.plot(L_I, c = L_col[i%len(L_col)])
#         #plt.plot(L_R, c = L_col[i%len(L_col)])
#     plt.scatter(L_max, L_tot, c = L_col[i%len(L_col)])

def vacation_factor(t):
    vacation_factor =1.0
    if 180<t%365<270 or t%365<10:
        vacation_factor = 0.2
    #vacation_factor =1.0
    return vacation_factor


# then add vaccination


def randomize(rate, N):
    events = 0
    for i in range(N):
        if random.random()<rate:
            events +=1
    return events

import numpy
def randomize(rate, N):
    return numpy.random.binomial(N, rate)


def simulation_sir_stochastic(beta, gamma, mu=1/50/365, noise_level=1, gamma_external=1e-3,
                              initial_I=1, population =5000, initial_S = 0, duration = 100):

    total_birth=0
    N = [population]
    I =[initial_I] 
    S =[initial_S] 
    R =[N[0] - I[0] - S[0]] 
    
    for t in range(duration):        
        
        new_infected = randomize(vacation_factor(t)*beta*I[-1]/N[-1], S[-1])
        
        new_infected += randomize(gamma_external, S[-1])
            
        new_recovered = randomize(gamma, I[-1])
        
        birth = randomize(mu, N[-1])
        total_birth += birth
                
        I.append(max(0, I[-1] + new_infected - new_recovered ))
        S.append(max(0, S[-1] - new_infected + birth))
        R.append(max(0, R[-1] + new_recovered -birth))
        N.append(I[-1]+S[-1]+R[-1])
        
    return S, I, R

R0 = 15.0
gamma = 1/5
beta = R0*gamma
#random.seed(1)
#for population in [1000]*5:
#plt.ylim(0, 2000)
#for population in [500, 1000, 2000, 10000, 1e5]:

# for population in [10000]:
#     L_S, L_I, L_R = simulation_sir_stochastic(beta, gamma, gamma_external=0.01/365, noise_level=1, population = population, duration=40*365)
#     plt.plot(L_I)

    #plt.plot(L_S)
    #plt.plot(L_R)
    #print(int(L_I[-1]*population))
    #plt.plot(L_R)
    #plt.plot(L_I)

#MC evaluation of  ?
#How often does it stay endemic? 
#number of outbreak
# plot as function of population
duration = 3*365
Lx=[]
Ly=[]
numtrials = 20

# for population in [100, 1000, 10000, 30000, 100000, 200000, 400000, 600000]:
#     total =0
#     for i in range(numtrials):
#         L_S, L_I, L_R = simulation_sir_stochastic(beta, gamma, \
#                                                   gamma_external=0.01/365, \
#                                                   noise_level=1, \
#                                                   population = population, 
#                                                   duration=duration)
#         empty_days = len([True for x in L_I if x==0])
#         total+=empty_days/duration
#     Lx.append(population) 
#     Ly.append(total/numtrials)
#     print(population)
# print(Ly)
# plt.scatter(Lx, Ly)


# rewrite with a function gamma(t)
# or gamma(t, I)

def simulation_sir_intervention(beta, gamma, 
                                time_intervention=30, fraction=0.5, 
                                population = 327e6, initial_I=1000, 
                                duration = 180):

    I = [initial_I]
    S = [population - initial_I]
    R = [0]   
   
    for t in range(duration):
        if t>time_intervention:
            effective_beta = beta*fraction
        else: 
            effective_beta = beta
            
        print(effective_beta)

        new_infected = effective_beta*I[-1]*S[-1] / population
        new_recovered = gamma * I[-1]
        I.append( I[-1] + new_infected - new_recovered )
        S.append( S[-1] - new_infected)
        R.append( R[-1] + new_recovered)
         
    return S, I, R


R0 = 2.5
gamma = 1/5
beta = R0*gamma

# S, I, R = simulation_sir(beta, gamma)
# plt.plot(I)
# S, I, R = simulation_sir_intervention(beta, gamma, 30, 0.5)
# plt.plot(I)

min_R0 = 2
max_R0 = 3
min_duration = 4
max_duration = 5



L_col =['tab:blue', "tab:orange", "tab:green", "tab:red", "tab:purple", "tab:brown", "tab:pink", "tab:gray", ]

# for i in range(20):
#     R0 = min_R0 + random.random()*(max_R0-min_R0)
#     duration = min_duration + random.random()*(max_duration-min_duration)
#     gamma = 1/duration
#     beta = R0*gamma

#     L_S, L_I, L_R  = simulation_sir(beta, gamma, duration = 200)
#     plt.plot(L_I, c=L_col[0])
#     #plt.plot(L_R, c=L_col[1])
#     L_S_i, L_I_i, L_R_i  = simulation_sir_intervention(beta, gamma, fraction_contact=0.5, duration = 200)
#     plt.plot(L_I_i, c=L_col[2])
#     #plt.plot(L_R_i, c=L_col[3])
#     L_S_i, L_I_i, L_R_i  = simulation_sir_intervention(beta, gamma, fraction_contact=0.33, duration = 200)
#     plt.plot(L_I_i, c=L_col[4])

#     #print ((1.0-L_R[-1])/(1.0-L_R_i[-1]))
    
    
    
def simulation_sir_trigerred_intervention(beta, gamma, 
                                          number_cases_intervention=100000, 
                                          fraction=0.5, 
                                          population = 327e6, initial_I=1000, 
                                          duration = 180):
    
    I = [initial_I]
    S = [population - initial_I]
    R = [0]   
   
    for t in range(duration):
        if I[-1]>number_cases_intervention:
            effective_beta = beta*fraction
        else: 
            effective_beta = beta
            
        new_infected = effective_beta*I[-1]*S[-1] / population
        new_recovered = gamma * I[-1]
        I.append( I[-1] + new_infected - new_recovered )
        S.append( S[-1] - new_infected)
        R.append( R[-1] + new_recovered)
         
    return S, I, R


R0 = 2.6
gamma = 1/5
beta = R0*gamma

#graph of total as function of contact reduction

# c=0
# for number_cases_intervention in [1e5, 1e6]:
#     for i in range(20):
#         L=[]
#         R0 = min_R0 + random.random()*(max_R0-min_R0)
#         duration = min_duration + random.random()*(max_duration-min_duration)
#         gamma = 1/duration
#         beta = R0*gamma
        
#         for fraction_contact in range(31):
#             frac = fraction_contact/100
#             L_S_i, L_I_i, L_R_i  = simulation_sir_trigerred_intervention(beta, gamma, 
#                                                               number_cases_intervention=number_cases_intervention, 
#                                                               fraction_contact=frac, duration = 200)
#             total_infected = (L_R_i[-1])*327e6
#             L.append(total_infected)
#         plt.plot(L, c=L_col[c])
#     c+=1


# graph of max 

# c=0
# for number_cases_intervention in [1e5, 1e6]:
#     for i in range(20):
#         L=[]
#         R0 = min_R0 + random.random()*(max_R0-min_R0)
#         duration = min_duration + random.random()*(max_duration-min_duration)
#         gamma = 1/duration
#         beta = R0*gamma
        
#         for fraction_contact in range(31):
#             frac = fraction_contact/100
#             L_S_i, L_I_i, L_R_i  = simulation_sir_trigerred_intervention(beta, gamma, 
#                                                               number_cases_intervention=number_cases_intervention, 
#                                                               fraction_contact=frac, duration = 200)
#             max_infected = max(L_R_i)*327e6
#             L.append(max_infected)
#         plt.plot(L, c=L_col[c])
#     c+=1

# compare strong vs weak intervention and early vs. slightly later timing. 
    
# L_S_i, L_I_i, L_R_i  = simulation_sir_trigerred_intervention(beta, gamma, 
#                                                              number_cases_intervention=1e5, 
#                                                              fraction_contact=0.5, duration = 200)
# plt.plot(L_I_i, c=L_col[0])

# L_S_i, L_I_i, L_R_i  = simulation_sir_trigerred_intervention(beta, gamma, 
#                                                              number_cases_intervention=1e6,
#                                                              fraction_contact=0.5, duration = 200)
# plt.plot(L_I_i, c=L_col[1])

# L_S_i, L_I_i, L_R_i  = simulation_sir_trigerred_intervention(beta, gamma, 
#                                                              number_cases_intervention=5e5,
#                                                              fraction_contact=0.3, duration = 200)
# plt.plot(L_I_i, c=L_col[2])

# L_S_i, L_I_i, L_R_i  = simulation_sir_trigerred_intervention(beta, gamma, 
#                                                              number_cases_intervention=5e6,
#                                                              fraction_contact=0.3, duration = 200)
# plt.plot(L_I_i, c=L_col[3])


# compare strong intervemtion for differnt virus parameters


# for i in range(5):
#     R0 = min_R0 + random.random()*(max_R0-min_R0)
#     duration = 5 #min_duration + random.random()*(max_duration-min_duration)
#     gamma = 1/duration
#     beta = R0*gamma
#     L_S_i, L_I_i, L_R_i  = simulation_sir_trigerred_intervention(beta, gamma, 
#                                                                   number_cases_intervention=5e5,
#                                                                   fraction_contact=0.3, duration = 200)
#     plt.plot(L_I_i, c=L_col[0])
#     plt.plot(L_R_i, c=L_col[1])
    
#     L_S_i, L_I_i, L_R_i  = simulation_sir_trigerred_intervention(beta, gamma, 
#                                                                   number_cases_intervention=5e6,
#                                                                   fraction_contact=0.3, duration = 200)
#     plt.plot(L_I_i, c=L_col[2])
#     plt.plot(L_R_i, c=L_col[3])



# promiscuous people
# do pairwise? 

def simulation_sir_variable_contact(beta, gamma, initial_I=100/327e6, duration = 100, n_bins=10):

    n_bins=3
    c=[0.1, 1, 1.9]

    
    I = [initial_I/n_bins]*n_bins
    S =  [1.0/n_bins]*n_bins
    R = [0]*n_bins
   
    L_S =[sum(S)] 
    L_I =[sum(I)] 
    L_R =[sum(R)] 
   
    
    for t in range(duration):
        for i in range(n_bins):            
            new_infected = 0
            for j in range(n_bins):
                contactness = c[i]*c[j]
                new_infected += beta * contactness * I[j]*S[i]
 
            new_recovered = gamma * I[i]

            I[i] = I[i] + new_infected - new_recovered
            S[i] = S[i] - new_infected
            R[i] = R[i] + new_recovered
        
        L_S.append(sum(S))
        L_I.append(sum(I))
        L_R.append(sum(R))
        
 
    return L_S, L_I, L_R

# L_S, L_I, L_R = simulation_sir(beta, gamma)
# plt.plot(L_I, c = L_col[0])
# plt.plot(L_R, c = L_col[0])
# L_S, L_I, L_R = simulation_sir_variable_contact(beta, gamma)
# plt.plot(L_I, c = L_col[1])
# plt.plot(L_R, c = L_col[1])

# add a version with a list of proportion of each contactedness



def simulation_sir_variable_contact_intervention(beta, gamma, initial_I=100/327e6, 
                                                 number_cases_intervention=100000, 
                                                 fraction_contact_good =0.5,
                                                 fraction_contact_bad =0.5,
                                                 duration = 100, n_bins=10):

    n_bins=3
    c=[0.1, 1, 1.9]

    
    I = [initial_I/n_bins]*n_bins
    S =  [1.0/n_bins]*n_bins
    R = [0]*n_bins
   
    L_S =[sum(S)] 
    L_I =[sum(I)] 
    L_R =[sum(R)] 
   
    intervention = False
    
    for t in range(duration):
        if not intervention and sum(I)*327e6>number_cases_intervention:
            intervention = True
            
        for i in range(n_bins):            
            new_infected = 0
            for j in range(n_bins):
                contactness = c[i]*c[j]
                if intervention:
                    if (i==n_bins-1) and (j==n_bins-1):
                        contactness *= fraction_contact_bad
                    else:
                        contactness *= fraction_contact_good
                        
                new_infected += beta * contactness * I[j]*S[i]
 
            new_recovered = gamma * I[i]

            I[i] = I[i] + new_infected - new_recovered
            S[i] = S[i] - new_infected
            R[i] = R[i] + new_recovered
        
        L_S.append(sum(S))
        L_I.append(sum(I))
        L_R.append(sum(R))
        
 
    return L_S, L_I, L_R


#graph of total as function of contact reduction

# c=0
# for factor_bad in [0.0, 0.5, 1.0]:
#     for i in range(20):
#         L=[]
#         R0 = min_R0 + random.random()*(max_R0-min_R0)
#         duration = min_duration + random.random()*(max_duration-min_duration)
#         gamma = 1/duration
#         beta = R0*gamma
        
#         for fraction_contact in range(31):
#             frac = fraction_contact/100
#             L_S_i, L_I_i, L_R_i  = simulation_sir_variable_contact_intervention(beta, gamma, 
#                                                               fraction_contact_good =frac,
#                                                               fraction_contact_bad =1-(1-frac)*factor_bad,
#                                                               duration = 200)
#             total_infected = (L_R_i[-1])*327e6
#             L.append(total_infected)
#         plt.plot(L, c=L_col[c])
#     c+=1



# do incubation + pre-infection


# do partial lockdown without vulnerable population


# for distancing, encapsulate as a function of date and case number

