#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Apr 21 16:04:28 2020

@author: fredodurand
"""

import random
import math





class Spatial_SIR(object):
    def __init__(self, x, y, population =1e3, R0 = 2.5, D = 5, initial_i = 0):
        self.x = x
        self.y = y
        self.population = population
        self.population = max(1, self.population)
        self.gamma = 1/D
        self.beta = R0*self.gamma
        self.I = [initial_i]
        self.R = [0]
        self.S =[population - initial_i]

    def distance_squared(self, v):
        dx, dy = v.x-self.x, v.y-self.y
        return dx*dx + dy*dy

    def step(self, t, vertices, dt=1, ):
        
        #external stuff?
        
        new_infected = self.beta*self.I[t]*self.S[t] / self.population
        new_recovered = self.gamma * self.I[t]
        for v in  vertices:
            if (v!=self):
                gravity = v.population*self.population/self.distance_squared(v)
                distant_new_infected = global_beta*gravity * self.S[t] * v.I[t]
                #print(gravity, distant_new_infected)                
                new_infected += distant_new_infected
        
        self.I.append(self.I[-1] + new_infected - new_recovered)
        self.S.append(self.S[-1] - new_infected)
        self.R.append(self.R[-1] + new_recovered)
        
    
    
def step(vertices):
    t = len(vertices[0].I)-1
    for v in vertices:
        v.step(t, vertices)
    

def make_random_vertices(N=200, max_pop=1000):
    vertices =[]
    for i  in range(N):
        pop = random.randint(1, max_pop)
        x, y = random.random(), random.random()
        vertices.append(Spatial_SIR(x, y, pop))
    return vertices



import json

def read_county(vertices, filename = 'COUNTY_25.json'):
    f = open(filename,)   
    print(filename)
    # returns JSON object as a dictionary 
    data = json.load(f) 
    # Closing file 
    f.close() 
    for n in data['nodes']:            
        vertices.append(Spatial_SIR(n['C_X'], n['C_Y'], n['TOTPOP']))



#vertices = make_random_vertices()
vertices =[]
# for i in range(2,4):
#     if (i<10):
#         filename = 'county/COUNTY_0'+str(i)+'.json'
#     else:
#         filename = 'county/COUNTY_'+str(i)+'.json'
#     read_county(vertices, filename)

#read_county(vertices, 'COUNTY_25.json')
read_county(vertices, 'COUSUB_25.json')
#read_county(vertices, 'TRACT_25.json')
#read_county(vertices, 'BG_25.json')

min_x = min([v.x for v in vertices])
max_x = max([v.x for v in vertices])
min_y = min([v.y for v in vertices])
max_y = max([v.y for v in vertices])
diameter = math.sqrt((max_x-min_x)**2+(max_y-min_y)**2) 

total_population = sum([v.population for v in vertices])

print(total_population)
global_beta = 1e-4 /total_population/total_population /diameter**2


vertices[1].I[0] = 100

import matplotlib.pyplot as plt
from matplotlib.animation import FuncAnimation



# Create new Figure and an Axes which fills it.
fig = plt.figure(figsize=(7, 7))

#split into original vertices and ones that just changed
v_plot = None

def draw_spatial(vertices):

    ax = fig.add_axes([0, 0, 1, 1], frameon=False)
    ax.set_xlim(min_x-diameter/20, max_x+diameter/20), ax.set_xticks([])
    ax.set_ylim(min_y-diameter/20, max_y+diameter/20), ax.set_yticks([])
    #fig = plt.figure()
    global v_plot
    N = len(vertices)
    L_x, L_y, L_c = [], [], []
    for i in range(N):
        v = vertices[i]
        L_x.append(v.x)
        L_y.append(v.y)
        ratio_I = v.I[-1]/v.population
        ratio_R = v.R[-1]/v.population  
        ratio_S = v.S[-1]/v.population  
        L_c.append((ratio_I*50,ratio_R,0.0,1.0))

    v_plot=ax.scatter(L_x, L_y, facecolors=L_c, s=diameter*10)     

    return v_plot

        
def update(frame_number):
    print(frame_number)
    step(vertices)
    plt.clf()
    draw_spatial(vertices)

# Construct the animation, using the update function as the animation director.
animation = FuncAnimation(fig, update, frames=300, interval=20, repeat = False)
plt.show()

# https://matplotlib.org/gallery/animation/basic_example_writer_sgskip.html