#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Apr 10 14:38:44 2020

@author: fredodurand
"""
import random
import matplotlib.pyplot as plt

random.seed(16)

class Individual (object):
    counter = 0
    def __init__(self, x, y, status='S'):
        self.index = Individual.counter
        Individual.counter += 1 
        self._status = [status]
        self.coordinates = (x, y)
        self.compliance = 1.0
    def get_status(self, t=-1):
        return self._status[t]
    def copy_previous_status(self):
        self._status.append(self._status[-1]);
        
    def set_status(self, new_status):
        self._status[-1]=new_status;
    def __str__(self):
        return self.get_status();
        

def make_grid(w, h):
    vertices =[]
    edges = []
    for y in range(h):
        for x in range(w):
            i = Individual(x, y)
            vertices.append(i)
            if (x>0): 
                edges.append((vertices[-2], i))
                edges.append((i, vertices[-2]))
            if (y>0): 
                edges.append((vertices[-w-1], i))
                edges.append((i, vertices[-w-1]))
                        
    return vertices, edges
import math

def make_hex(w, h):
    vertices =[]
    edges = []
    for y in range(h):
        for x in range(w):
            
            i = Individual(x+math.cos(math.pi/3)*(y%2), y*math.sin(math.pi/3))
            vertices.append(i)
            if (x>0): 
                edges.append((vertices[-2], i))
                edges.append((i, vertices[-2]))
            if (y>0): 
                edges.append((vertices[-w-1], i))
                edges.append((i, vertices[-w-1])) 
                if y%2==0:
                    if (x>0):
                        edges.append((vertices[-w-2], i))
                        edges.append((i, vertices[-w-2])) 
                else: 
                    if x<w-1:
                        edges.append((vertices[-w], i))
                        edges.append((i, vertices[-w])) 
    return vertices, edges


def make_hex_plus_distant_links(w, h, exponent=4, base_rate=0.1):
    vertices, edges = make_hex(w, h)
    for i in vertices:
        for j in vertices:
            dx = i.coordinates[0]-j.coordinates[0]     
            dy = i.coordinates[1]-j.coordinates[1]     
            d2 = dx*dx+dy*dy
            if d2>1.1:
                if random.random()<base_rate/d2**(exponent/2):
                    edges.append((i, j))
                    edges.append((j, i))
    return vertices, edges

L_I, L_S, L_R = [], [], []
       

def simulate_one_step(vertices, edges, beta, gamma, dt):
    current_time = len(vertices[0]._status)-1
    L_contagion_edges = []

    # count people
    I, S, R=0,0,0
    for i in vertices:
        if i.get_status()=='I': I+=1
        if i.get_status()=='S': S+=1
        if i.get_status()=='R': R+=1
    L_I.append(I)
    L_S.append(S)
    L_R.append(R)
    for i in vertices:
        i.copy_previous_status()   
        
    for e in edges:
        if e[0].get_status(current_time)=='I' and e[1].get_status(current_time)=='S':
            if random.random()<beta*dt:
                e[1].set_status('I')
                L_contagion_edges.append(e)
    for i in vertices:
        if i.get_status(current_time)=='I' and random.random()<gamma*dt:
            i.set_status('R')
    return L_contagion_edges
   
w = 20
h = 20 

N = 100
contacts = 2
gamma = 1/10
#beta = R0*gamma*N/contacts
beta = 0.055

#vertices, edges = make_grid(w, h)
#vertices, edges = make_hex(w, h)
vertices, edges = make_hex_plus_distant_links(w, h, 3, 0.1)

vertices[w*h//2+w//2].set_status('I')


# plt.plot(L_S)
# plt.plot(L_I)
# plt.plot(L_R)
from matplotlib.animation import FuncAnimation

# Create new Figure and an Axes which fills it.
fig = plt.figure(figsize=(7, 7))

#split into original vertices and ones that just changed
v_plot, ce_plot = None, None


def draw_graph(vertices, edges, L_contagion_edges):
    ax = fig.add_axes([0, 0, 1, 1], frameon=False)
    ax.set_xlim(-1, w), ax.set_xticks([])
    ax.set_ylim(-1, h), ax.set_yticks([])
    #fig = plt.figure()
    global v_plot
    global ce_plot
    N = len(vertices)
    L_x, L_y, L_c = [], [], []
    for i in range(N):
        ind = vertices[i]
        L_x.append( ind.coordinates[0])
        L_y.append(ind.coordinates[1])
        if vertices[i].get_status()=='I':
            L_c.append((1.0,0.0,0.0,1.0))
        elif vertices[i].get_status()=='R':
            L_c.append((0.0,1.0,0.0,1.0))
        else: 
            L_c.append((0.5,0.5,0.5,1.0))
    v_plot=ax.scatter(L_x, L_y, facecolors=L_c, s=1000/w, zorder = 3)     
    if True:
        L_x_e =[]
        L_y_e =[]
        for e in edges:
            L_x_e.append(L_x[e[0].index])
            L_x_e.append( L_x[e[1].index])
            L_x_e.append(None)
            L_y_e.append(L_y[e[0].index])
            L_y_e.append(L_y[e[1].index])
            L_y_e.append(None)
        ax.plot(L_x_e, L_y_e, color=(0.9, 0.9, 0.9))                
    L_x_e =[]
    L_y_e =[]
    for e in L_contagion_edges:
        L_x_e.append(L_x[e[0].index])
        L_x_e.append( L_x[e[1].index])
        L_x_e.append(None)
        L_y_e.append(L_y[e[0].index])
        L_y_e.append(L_y[e[1].index])
        L_y_e.append(None)
    ce_plot=ax.plot(L_x_e, L_y_e, color=(1.0, 0.0, 0.0))                

    return v_plot, ce_plot



def update(frame_number):
    print(frame_number)
    L_contagion_edges = []
    if frame_number == 0:
        draw_graph(vertices, edges, L_contagion_edges)
    else:
        L_contagion_edges = simulate_one_step(vertices, edges, beta, gamma, 1.9)
        plt.clf()
        draw_graph(vertices, edges, L_contagion_edges)

# Construct the animation, using the update function as the animation director.
animation = FuncAnimation(fig, update, frames=300, interval=20, repeat = False)
plt.show()

# https://matplotlib.org/gallery/animation/basic_example_writer_sgskip.html