#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Apr  9 16:46:19 2020

@author: fredodurand
"""

#make graph dictionary
# from vertex to list of edges
# then only transmit from I vertices

# make more synchronous. right now I modify vertices in place. 
# Maybe just keep track of full history for each vertex? 
# also makes it easier to decouple animation from simulation



# https://towardsdatascience.com/animations-with-matplotlib-d96375c5442c

import random
import matplotlib.pyplot as plt

class individual (object):
    counter = 0
    def __init__(self, status='S'):
        self.index = individual.counter
        individual.counter += 1 
        self._status = status

    def get_status(self):
        return self._status
    def set_status(self, new_status):
        self._status = new_status;
    def __str__(self):
        return self.get_status();
    
def make_graph(n_individuals, average_edges_per_person):
    vertices = []
    for i in range(n_individuals):
        vertices.append(individual())
    edges = []
    for i in range(n_individuals):
        edges.append((vertices[i], vertices[(i+1)%n_individuals]))
        for j in range(average_edges_per_person-1):
            k = random.randint(0,n_individuals-1)
            edges.append((vertices[i], vertices[k]))
    return vertices, edges
         
L_I, L_S, L_R = [], [], []
       

def simulate_one_step(vertices, edges, beta, gamma, dt):
    # count people
    L_contagion_edges = []
    I, S, R=0,0,0
    for i in vertices:
        if i.get_status()=='I': I+=1
        if i.get_status()=='S': S+=1
        if i.get_status()=='R': R+=1
    L_I.append(I)
    L_S.append(S)
    L_R.append(R)

    for e in edges:
        if e[0].get_status()=='I' and e[1].get_status()=='S':
            if random.random()<beta*dt:
                e[1].set_status('I')
                L_contagion_edges.append(e)
    for i in vertices:
        if i.get_status()=='I' and random.random()<gamma*dt:
            i.set_status('R')
    return L_contagion_edges
    

N = 100
contacts = 2
gamma = 1/10
#beta = R0*gamma*N/contacts
beta = 0.25
vertices, edges = make_graph(N, contacts)
vertices[0].set_status('I')


# plt.plot(L_S)
# plt.plot(L_I)
# plt.plot(L_R)
import math
from matplotlib.animation import FuncAnimation

dt = 0.5

# Create new Figure and an Axes which fills it.
fig = plt.figure(figsize=(7, 7))
ax = fig.add_axes([0, 0, 1, 1], frameon=False)
ax.set_xlim(-1.1, 1.1), ax.set_xticks([])
ax.set_ylim(-1.1, 1.1), ax.set_yticks([])

def draw_graph(vertices, edges, L_contagion_edges):
    #fig = plt.figure()
    N = len(vertices)
    L_x, L_y, L_c = [], [], []
    for i in range(N):
        theta = i/N*2*3.14159
        L_x.append( math.cos(theta))
        L_y.append(math.sin(theta))
        if vertices[i].get_status()=='I':
            L_c.append((1.0,0.0,0.0,1.0))
        elif vertices[i].get_status()=='R':
            L_c.append((0.0,1.0,0.0,1.0))
        else: 
            L_c.append((0.0,0.0,0.0,1.0))
    ax.scatter(L_x, L_y, facecolors=L_c, s=100, zorder = 3)
   
    for e in edges:
        L_x_e = [L_x[e[0].index], L_x[e[1].index]]
        L_y_e = [L_y[e[0].index], L_y[e[1].index]]
        ax.plot(L_x_e, L_y_e, color=(0.5, 0.5, 0.5))                
    for e in L_contagion_edges:
        L_x_e = [L_x[e[0].index], L_x[e[1].index]]
        L_y_e = [L_y[e[0].index], L_y[e[1].index]]
        ax.plot(L_x_e, L_y_e, color=(1.0, 0.0, 0.0))                

def update(frame_number):
    print(frame_number)
    L_contagion_edges = []
    if frame_number > 0:             
        L_contagion_edges = simulate_one_step(vertices, edges, beta, gamma, 0.7)
    draw_graph(vertices, edges, L_contagion_edges)

# Construct the animation, using the update function as the animation director.
animation = FuncAnimation(fig, update, frames=100, interval=300)
plt.show()



#L_S, L_I, L_R = simulate(vertices, edges, beta, gamma)
